#!/usr/bin/env python
#
# Print the stats from an iwalk output file

import sys,os,shelve

sys.path.append(os.getenv("HOME")+"/slg/src/python")

from histogram2d import histogram2d
from histogram import histogram
from statbag import statbag
from table import table
import re

just_parts = False
driveid_re = re.compile("(\d+)[.]")

class iwalkpart:
    """ Object for holding information about each iwalk partition"""
    import re
    fstype_re = re.compile("ftype.*\(([^\)]*)\)")
    def __init__(self,filename,line):
        self.filename = filename
        self.props    = {}
        self.files   = []
        for w in line.split(" "):
            terms = w.split("=")
            if(len(terms)==2):
                setattr(self,terms[0],terms[1])
        m = self.fstype_re.search(line)
        if(m):
            self.fstype = m.group(1)
    def __str__(self):
        return self.filename
    def __repr__(self):
        return self.filename

class iwalkfile:
    """ Object for holding information about each file """
    def __str__(self):
        return "%s (size:%d frag:%d)" % (self.filename,self.size,self.fragments)
    def __repr__(self):
        return "%s (size:%d frag:%d)" % (self.filename,self.size,self.fragments)
    def ext(self):
        pos = self.filename.rfind('.')
        if(len(self.filename)-pos>=6): return ""
        r = self.filename[pos+1:].lower()
        if(r=='jpeg'): return 'jpg'
        if(r=='jfif'): return 'jpg'
        return r
        

def process_partition(filename,line):
    line = line.replace(": ","=")
    ret  = iwalkpart(filename,line)
    return ret
    

def readfile(filename):
    """ Read two arrays: the first of the partitions, the second of the files"""

    driveid = int(driveid_re.search(filename).group(1))

    import re
    getint_re = re.compile("\d+")
    def getint(s):
        return int(getint_re.search(s).group(0))
    parts = []                            # list of what we will return
    files = []
    image_file = ""
    this_part = False
    this_file = False
    infiles = False
    for line in open(filename,"r"):
        if(line.startswith("Cannot")): continue
        if(line.startswith("#")): continue
        if(line[0]=='='):
            image_file = ""
            continue
        line = line.strip()             # remove whitespace
        if(len(line)==0):               # blank line; time to move to new record
            infiles = True
            this_file = False           # reset it
            continue
        if(line.startswith("Partition")):
            image_partition = line[line.find(':')+1:]
            this_part = process_partition(filename,image_partition)
            parts.append(this_part)
            continue

        if(line.startswith("Filename:")): infiles=True
        if infiles==False: continue     # not in the files yet

        if(this_file==False):
            this_file = iwalkfile()
            this_file.filename = line
            this_part.files.append(this_file)
            files.append(this_file)

        if(line.startswith("Invalid argument")): continue
        if(line.startswith("Error reading image file")): continue
        if(line.startswith("Imagefile:")): this_file.filename = line[11:];continue
        if(line.startswith("Filename:")):
            this_file.filename = line[10:];
            if just_parts: break
            continue
        if(line.startswith("Size:")): this_file.size = int(line[6:]);continue
        if(line.startswith("Fragments:")): this_file.fragments = int(line[11:]);continue
        if(line.startswith("Block runs:")): this_file.block_runs = line[12:];continue
        if(line.startswith("Sector runs:")): this_file.sector_runs = line[13:];continue
	#if(line.startswith("Status:")): this_file.status = line[8:];continue
        #if(line.startswith("mtime:")): this_file.mtime = getint(line);continue
        #if(line.startswith("atime:")): this_file.atime = getint(line);continue
        #if(line.startswith("ctime:")): this_file.ctime = getint(line);continue
        #if(line.startswith("Fragmentation:")): this_file.fragmentation = line[15:];continue
        #if(line.startswith("MD5:")): this_file.md5 = line[5:];continue
        if(line.startswith("maxrss:")): continue # 
        if(line.startswith("inputs:")): continue # 

        # legacy: first line is image line
        if(image_file==""): image_file = line;continue;
        #print "unknown line:",line
    return (parts,files)

def fragmentation_table(ifiles):
    frag_hist = histogram2d()
    for i in ifiles:
        frag_hist.add(i.fragments,i.size)
    tab = table()
    tab.header = "File Fragmentation"
    tab.col_headings = ["# of fragments","# of files",'Average Filesize']
    frag_hist.statcol = 'iaverage'
    return frag_hist.typeset(t=tab,sorted=True)

    data = []
    for i in frag_hist.topn(20):
        count = i[0]
        for key in i[1]:
            data.append([key,count])
    data.sort()
    tab.data = data
    return tab


def process_files(files):
    from table import table
    count = 0
    drive_files = {}                         # index of drives
    all_parts  = []
    all_files = []
    fstypeh = histogram2d()
    for fn in files:
        count += 1
        driveid = int(driveid_re.search(fn).group(1))
        print "%s... (Drive %d)" % (fn,driveid)

        (parts,files) = readfile(fn)
        all_parts.extend(parts)
        all_files.extend(files)

        # Process the partitions
        if(parts == []):
            fstypeh.add("(unrecognized)",0)
        else:
            for p in parts:
                fstypeh.add(p.fstype,int(p.block_count)*int(p.block_size)/1000000)

    fstypes = fstypeh.histogram.unique_names()
    fstypes.sort()


    #
    # Information about fragmentation patterns
    #
    print "Fragmentation pattern histogram, by file system and file type:"
    for fstype in fstypes:
        for ftype in ['jpg','pdf','doc','txt']:
            len1stats = statbag()
            len2stats = statbag()
            delta_hist = histogram()
            delta_re = re.compile("(\d+)\-?(\d+)? ?(\d+)\-?(\d+)?")
            for i in filter( (lambda(f): f.ext()==ftype and f.fragments==2),all_files):
                runs = False
                if(hasattr(i,'block_runs')): runs = i.block_runs
                if(hasattr(i,'sector_runs')): runs = i.sector_runs
                if not runs: continue
                m = delta_re.search(runs)
                r = []
                for j in range(1,5):
                    try:
                        r.append(int(m.group(j)))
                    except TypeError:
                        r.append(int(m.group(j-1)))

                len1 = r[1] - r[0] + 1
                len2 = r[3] - r[2] + 1
                delta = r[2]-r[1]
                
                len1stats.addx(len1)
                len2stats.addx(len2)
                delta_hist.add(delta)

            if len1stats.count()>0:
                print "\n\n"
                print "fstype:",fstype,"  ftype:",ftype
                print "len1 average: %f stddev: %f" % (len1stats.average(),len1stats.stddev())
                print "len2 average: %f stddev: %f" % (len2stats.average(),len2stats.stddev())
                print "delta average: %f" % delta_hist.average()
                print "delta histogram:"
                delta_hist.print_top(10)
    sys.exit(0)


    #
    # Typeset information about file extensions
    #
    hist_exts = histogram2d()
    hist_exts.topn = 20
    for i in all_files:
        if i.size>0 and i.fragments>0: hist_exts.add(i.ext(),i.size)
    tab = table()
    tab.header     = "File extension popularity and average size (suppressing 0-len files)"
    tab.col_headings = ['Ext','Count','Average Size','Max','Std Dev']
    tab.omit_row = [[0,'']]
    hist_exts.statcol = ['iaverage','maxx','istddev']
    print hist_exts.typeset(t=tab)

    hist_exts = histogram2d()
    hist_exts.topn = 20
    for i in all_files:
        if i.fragments>0: hist_exts.add(i.ext(),i.fragments)
    tab = table()
    tab.header     = "Fragmentation by file extension (suppressing files with 0 fragments)"
    tab.col_headings = ['Ext','Count','Avg Fragments','Max','Std Dev']
    tab.omit_row = [[0,'']]
    hist_exts.statcol = ['average','maxx','stddev']
    print hist_exts.typeset(t=tab)

    print "==========================="


    #
    # Typeset the File Systems on Drives table
    #

    tab = table()
    tab.header     = "File Systems on Drives"
    tab.col_headings = ["FS Type","Drives","MBytes"]
    tab.col_totals = [1,2]
    fstypeh.statcol = 'sumx'
    print fstypeh.typeset(t=tab)

    #
    # Typeset overall fragmentation stats
    #

    print fragmentation_table(all_files)

    #
    # Typeset the Fragmentation per filesystem type
    #

    for fstype in fstypes:
        if fstype=='(unrecognized)': continue
        print fstype,"Partitions:"

        def isfstype(x): return x.fstype==fstype
        these_parts = filter(isfstype,all_parts)
        these_files = []
        for part in these_parts:
            these_files.extend(part.files)
        print fragmentation_table(these_files)

if(__name__=="__main__"):
    import re,getopt
    def usage():
        print "usage: %s [options] [iwalkfiles]" % sys.argv[0]
        print "   -l, --load == load the database from the walk files"
        sys.exit(0)

    try:
        opts,args = getopt.getopt(sys.argv[1:],"b",["build"])
    except getopt.GetoptError:
        usage()
        sys.exit(2)
    for o,a in opts:
        if o in ("-h"):
            usage()
        if o in ("-b","--build"):
            build()
            
    #d = shelve.open("stats")

    process_files(sys.argv[1:])
